/*
  netype (Sample NTS Client) animate.c

  Copyright (C) 2000 Raiko FUNAKAMI

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <stdio.h>
#include "animate.h"
#include "status.h"
#include "gekic.h"


/*
  Υ⡼󤬽λ٤Ȥ֤ͤǼ

  FALSE : λ
  TRUE  : ³
*/
gboolean motion_neko_leave(type_actor *actor, gpointer data)
{
  const int STEP_X = -80;
  GSList *node = NULL;
  if(actor->x < -100){
    return FALSE;
  }

  
  actor->x += STEP_X;
  for(node = actor->children; node ; node = node->next){
    type_question *q = (type_question *)node->data;
    q->x += STEP_X;
  }

  return TRUE;
}

gboolean motion_neko_steal(type_actor *actor, gpointer data)
{
  const int STEP_X = 80;
  GSList *node = NULL;
  if(actor->x > 700){
    return FALSE;
  }

  
  actor->x += STEP_X;
  for(node = actor->children; node ; node = node->next){
    type_question *q = (type_question *)node->data;
    q->x += STEP_X;
  }

  return TRUE;
}

gboolean motion_neko_nomove(type_actor *actor, gpointer data)
{
  return TRUE;
}



static void get_width_height(gchar **xpm, int *width, int *height)
{
  sscanf(xpm[0], "%d %d", width, height);
}

type_actor* type_actor_new(){
  type_actor* actor = (type_actor*)g_malloc(sizeof(type_actor));

  actor->seq = 0;
  actor->n_sequences = 0;

  actor->x = 0;
  actor->y = 0;
  actor->height = 0;
  actor->width = 0;

  actor->repeat = 1;
  actor->repeat_counter = 0;

  actor->sprites = NULL;
  actor->m_func = NULL;

  actor->children = NULL;

  return actor;
}
void type_actor_free(type_actor* actor){
  g_free(actor);
}

void load_pixmaps(GtkWidget *widget, type_actor *actor, type_sprite *sprites)
{
  int i = 0;

  while(sprites[i].xpm_data){
    sprites[i].pixmap = 
      gdk_pixmap_create_from_xpm_d(widget->window,
				   &sprites[i].mask,
				   NULL,
				   sprites[i].xpm_data);
    i++;
  }

  get_width_height(sprites[0].xpm_data, &actor->width, &actor->height);

  actor->seq = 0;
  actor->n_sequences = i;
  actor->sprites = sprites;
  actor->repeat = 1;
  actor->repeat_counter = 0;
}

type_actor* type_actor_dup(type_actor* a)
{
  type_actor* new_actor = type_actor_new();

  new_actor->seq         = a->seq;
  new_actor->n_sequences = a->n_sequences;

  new_actor->x = a->x;
  new_actor->y = a->y;
  new_actor->height = a->height;
  new_actor->width  = a->width;

  new_actor->repeat         = a->repeat;
  new_actor->repeat_counter = a->repeat_counter;

  new_actor->sprites = a->sprites;
  new_actor->m_func  = a->m_func;
  
  /* ҶʣϥݡȤƤʤ */
  new_actor->children = NULL;

  return new_actor;
}


void motion_set(type_actor *actor, gint repeat, MotionFunc func)
{
  actor->repeat = repeat;
  actor->repeat_counter = 0;
  actor->m_func = func;
}

/*
  pixmap : ֥Хåե GdkPixmap
  gc     : եåƥ
  d_frame: ô֤Υ޿( 1/30ä˰ٸƤФΤȤƺ)

  ֤
    FALSE : ˥᡼νλ
    TRUE  : ˥᡼Ϸ³
*/
gboolean draw_sprite(GdkPixmap *pixmap, GdkGC *gc, type_actor *actor)
{
  /*
    ˥᡼󤬰Υե졼ᤷ
     ԡȤȤꤹ
  */
  if(actor->seq >= actor->n_sequences){
    if(actor->repeat > 0 && actor->repeat_counter < actor->repeat)
      ++actor->repeat_counter;
    else{
      actor->repeat_counter = 0;
      if(actor->repeat > 0){
	return FALSE;
      }
    }

    actor->seq = 0;
  }

  /* SET MASK */
  gdk_gc_set_clip_mask(gc, actor->sprites[actor->seq].mask);
  gdk_gc_set_clip_origin(gc, actor->x, actor->y);

  /* DRAW Pixmap */
  gdk_draw_pixmap(pixmap, 
		  gc,
		  actor->sprites[actor->seq].pixmap,
		  0, 0,
		  actor->x, actor->y,
		  actor->width, actor->height);

  /* Clear MASK */
  gdk_gc_set_clip_mask(gc, NULL);


  /* Next Frame */
  ++actor->seq;

  /* ⡼ؿθƽФ */
  return actor->m_func(actor, NULL);
}



GSList* append_animate(GSList *list, type_actor **actor) 
{
  return g_slist_append(list, *actor);
}

GSList* remove_animate(GSList *list, type_actor **actor) 
{
  list = g_slist_remove(list, *actor);
  type_actor_free(*actor);

  //g_print("list = %p\n", list);
  return list;
}


/*
  BUGS 
     ƻҴط2Τߤθ Ҥ type_question˸
*/
GSList* draw_animation(GSList *list, GdkPixmap *pixmap, GdkGC *gc, gpointer data)
{
  type_status *status = (type_status *)data;
  GSList *node = NULL;
  GSList *remove_list = NULL;
  GSList *child = NULL;
  type_actor *actor = NULL;

  for(node = list; node; node = node->next){
    actor = (type_actor *)node->data;

    /* ҤΥ˥᡼ */
    for(child = actor->children; child ; child = child->next){
      type_question *q = (type_question *)child->data;
      draw_question(pixmap, gc, q);
    }

    /* ƤΥ˥᡼ */
    if(!draw_sprite(pixmap, gc, actor)){
      remove_list = g_slist_append(remove_list, actor);
    }
  }


  // λ˥᡼
  for(node = remove_list; node; node = node->next){
    actor = (type_actor *)node->data;

    // Ҷ type_question ȸꤷƺ
    for(child = actor->children; child; child = child->next){ 
      type_question* q = (type_question *)child->data;
      type_question_free(q);
      //g_print("free child animation\n");
    }

    // ʬȤ
    list = remove_animate(list, &actor);
    //g_print("free animation\n");

    status->qlist_drawn = FALSE;
  }

  return list;
}



