/*
  netype (Sample NTS Client) callbacks.c

  Copyright (C) 2000 Raiko FUNAKAMI

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <gtk/gtk.h>
#include <gdk/gdkkeysyms.h>
#include <gdk_imlib.h>
#include <stdlib.h>
#include <ctype.h>

#include "callbacks.h"
#include "interface.h"
#include "support.h"

#include "gekic.h"



extern type_status *client_status;

extern GdkImlibImage *im_wall;
extern GdkPixmap *pix_darea;
extern GdkPixmap *pix_wall;
extern GdkPixmap *pix_board[3];


extern GdkImlibImage *im_gover;
extern GdkPixmap *pix_gover;


/*
  Windows
*/
extern GtkWidget *win_main;
extern GtkWidget *win_setting;
extern GtkWidget *win_start_error_dialog;
extern GtkWidget *win_info;
extern GtkWidget *win_connection_error_dialog;
extern GtkWidget *win_wait;
extern GtkWidget *win_gover;

/*
  Error Dialog Label
*/
extern GtkWidget *label_cerror_dialog;

gboolean
on_win_main_key_press_event            (GtkWidget       *widget,
                                        GdkEventKey     *event,
                                        gpointer         user_data)
{
  static GString* buffer = NULL;
  static GString* tmp = NULL;

  if(!buffer) buffer = g_string_new("");
  else g_string_assign(buffer, "");

  if(!tmp) tmp = g_string_new("");
  else g_string_assign(tmp, "");

  /*
    Check Game Start 
  */
  if(client_status->start != TRUE)
    return FALSE;


  if(!client_status->pressed_keys)
    client_status->pressed_keys = g_string_new("");

  if(event->keyval == GDK_Return){
    g_string_append(buffer, "KILL ");
    g_string_append(buffer, client_status->pressed_keys->str);
    net_writeln(client_status->net_info, buffer->str);

    g_string_assign(client_status->pressed_keys, "");
  }
  else if(event->keyval == GDK_Delete || 
	  event->keyval == GDK_BackSpace){
    g_string_assign(tmp, client_status->pressed_keys->str);
    tmp->str[tmp->len - 1] = '\0';
    g_string_assign(client_status->pressed_keys, tmp->str);
  }
  else{
    if(GDK_space <= event->keyval && event->keyval <= GDK_z){
      gchar tmp[2];
      tmp[0] = (gchar)(event->keyval);
      tmp[1] = '\0';
      g_string_append(client_status->pressed_keys, tmp);
    }
  }

  return FALSE;
}


gboolean
delete_event                           (GtkWidget       *widget,
                                        GdkEvent        *event,
                                        gpointer         user_data)
{
  
  return FALSE;
}


gboolean
destroy_event                          (GtkWidget       *widget,
                                        GdkEvent        *event,
                                        gpointer         user_data)
{
  net_close(client_status->net_info);
  gtk_main_quit();

  return FALSE;
}


void
on_btn_setting_clicked                 (GtkButton       *button,
                                        gpointer         user_data)
{
  gtk_widget_show(win_setting);
}



/*
  Connect to  Server
*/
void
on_btn_start_clicked                   (GtkButton       *button,
                                        gpointer         user_data)
{
  char *p = NULL;
  gboolean check_flag = TRUE;

  /* 
     Check Status
  */
  if(client_status->port < 0 || client_status->port > 65546)
    check_flag = FALSE;

  if(client_status->user_name->str[0] == '\0')
    check_flag = FALSE;

  if(client_status->host_name->str[0] == '\0')
    check_flag = FALSE;
  
   
  if(check_flag != TRUE){
    gtk_widget_show(win_start_error_dialog);
    return;
  }


  /*
    Set User Name
  */
  p = client_status->user_name->str;
  while(*p){
    /*
      if(!isalnum(*p) && !ispunct(*p)){
    */
    if(isspace(*p) || iscntrl(*p) ){
      *p = '_';
    }
    ++p;
  }


  /*
    Set Network
  */
  if(client_status->net_info && client_status->net_info->socket > 0 ){
    g_print("still connect!\n");
    return;
  }


  client_status->net_info = connect_to_server(client_status->host_name->str,
					      client_status->user_name->str, 
					      client_status->passwd->str,
					      client_status->port);

  if(!client_status->net_info){
    gtk_label_set_text(GTK_LABEL(label_cerror_dialog), "CONNECT CLOSED ERROR!");
    gtk_widget_show(win_connection_error_dialog);
    return;
  }

  /*
    Set Flags
   */
  client_status->end = FALSE;

  /*
    Network CallBack Function
  */
  client_status->net_info->read_tag = gdk_input_add(client_status->net_info->socket, 
						     GDK_INPUT_READ,
						     read_from_server,
						     client_status);

  /*
  client_status->net_info->write_tag = gdk_input_add(client_status->net_info->socket,
						     GDK_INPUT_WRITE,
						     write_to_server,
						     client_status);

  */

  /*
    Show Wait Window
  */
  gtk_widget_show(win_wait);
  
  return;
}


void
on_btn_quit_clicked                    (GtkButton       *button,
                                        gpointer         user_data)
{
  destroy_event(GTK_WIDGET(button), NULL, user_data);
}

void
on_btn_info_clicked                    (GtkButton       *button,
                                        gpointer         user_data)
{
  gtk_widget_show(win_info);
}



gboolean
on_darea_configure_event               (GtkWidget       *widget,
                                        GdkEventConfigure *event,
                                        gpointer         user_data)
{
  /*  
      Set Drawing Area Pixmap
  */
  if(pix_darea){
    gdk_pixmap_unref(pix_darea);
  }
  pix_darea = gdk_pixmap_new(widget->window, 
			  widget->allocation.width,
			  widget->allocation.height,
			  -1);


  /*
    Set BackGround Pixmap
  */
  if(pix_wall){
    gdk_pixmap_unref(pix_wall);
  }
  gdk_imlib_render(im_wall, widget->allocation.width, widget->allocation.height);
  pix_wall = gdk_imlib_move_image(im_wall);


  return FALSE;
}


gboolean
on_darea_expose_event                  (GtkWidget       *widget,
                                        GdkEventExpose  *event,
                                        gpointer         user_data)
{
  /* 
     Draw Drawing Area Pixmap
  */
  gdk_draw_pixmap(widget->window,
		  widget->style->fg_gc[GTK_WIDGET_STATE(widget)],
		  pix_darea,
		  event->area.x, event->area.y,
		  event->area.x, event->area.y,
		  event->area.width, event->area.height);

  gdk_flush();

  return FALSE;
}

void
on_btn_setting_ok_clicked              (GtkButton       *button,
                                        gpointer         user_data)
{
  char *port_str = NULL;
  char *p = NULL;
  GtkEntry *entry_uname = GTK_ENTRY(((GtkWidget **)user_data)[0]);
  GtkEntry *entry_passwd = GTK_ENTRY(((GtkWidget **)user_data)[1]);
  GtkEntry *entry_hname = GTK_ENTRY(((GtkWidget **)user_data)[2]);
  GtkEntry *entry_port = GTK_ENTRY(((GtkWidget **)user_data)[3]);

  /* 
     Setting Status
  */
  g_string_assign(client_status->user_name, gtk_entry_get_text(entry_uname));
  g_string_assign(client_status->passwd, gtk_entry_get_text(entry_passwd));
  g_string_assign(client_status->host_name, gtk_entry_get_text(entry_hname));

  p = port_str = gtk_entry_get_text(entry_port);
  while(*p){
    if(!isdigit(*p++)){
      port_str[0] = '\0';
      break;
    }
  }
  if(port_str[0] != '\0'){
    client_status->port = atoi(port_str);
  }else{
    client_status->port = -1;
  }


  g_print("USER NAME = %s\n", client_status->user_name->str);
  g_print("PASSWD    = %s\n", client_status->passwd->str);
  g_print("JPST NAME = %s\n", client_status->host_name->str);
  g_print("PORT      = %d\n", client_status->port);

  gtk_widget_hide(win_setting);
}


void
on_btn_setting_cancel_clicked          (GtkButton       *button,
                                        gpointer         user_data)
{
  g_print("USER NAME = %s\n", client_status->user_name->str);
  g_print("PASSWD    = %s\n", client_status->passwd->str);
  g_print("JPST NAME = %s\n", client_status->host_name->str);
  g_print("PORT      = %d\n", client_status->port);

  gtk_widget_hide(win_setting);
}


void
on_btn_info_ok_clicked                 (GtkButton       *button,
                                        gpointer         user_data)
{
  gtk_widget_hide(win_info);
}




void 
on_btn_connection_error_dialog_ok_clicked(GtkButton       *button, 
					  gpointer         user_data)
{
  gtk_widget_hide(win_connection_error_dialog);
}


gboolean
on_darea_game_over_configure_event     (GtkWidget       *widget,
                                        GdkEventConfigure *event,
                                        gpointer         user_data)
{
  /*  
      Set Drawing Area Pixmap
  */
  /*
  gdk_imlib_render(im_game_over, widget->allocation.width, widget->allocation.height);
  if(pix_game_over){
    gdk_pixmap_unref(pix_game_over);
  }
  pix_game_over = gdk_imlib_move_image(im_game_over);
  */
  return FALSE;
}


gboolean
on_darea_game_over_expose_event        (GtkWidget       *widget,
                                        GdkEventExpose  *event,
                                        gpointer         user_data)
{
  /* 
     Draw Drawing Area Pixmap
  */

  gdk_draw_pixmap(widget->window,
		  widget->style->fg_gc[GTK_WIDGET_STATE(widget)],
		  pix_gover,
		  event->area.x, event->area.y,
		  event->area.x, event->area.y,
		  event->area.width, event->area.height);

  gdk_flush();

  return FALSE;
}

void
on_btn_game_over_ok_clicked        (GtkButton       *button,
				    gpointer         user_data)

{
  gtk_widget_hide(win_gover);
}



void
on_btn_start_error_dialog_ok_clicked   (GtkButton       *button,
                                        gpointer         user_data)
{
  gtk_widget_hide(win_start_error_dialog);
}

