/*
  netype (Sample NTS Client) draw.c

  Copyright (C) 2000 Raiko FUNAKAMI

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <stdio.h>
#include <gtk/gtk.h>
#include <gdk_imlib.h>
#include "gekic.h"


  
#define QAREA_MAX 480 


extern type_status *client_status;
extern GdkFont *font_question;
extern type_actor act_tmp_carrycat;
extern type_actor act_tmp_stealcat;


extern GdkImlibImage *im_board[3];
extern GdkPixmap *pix_darea;
extern GdkPixmap *pix_wall;
extern GdkPixmap *pix_board[3];
extern GdkBitmap *bit_board[3];


GSList *anime_list = NULL;

void draw_font_init(){
  /*
  font_question = gdk_fontset_load("-sony-fixed-medium-r-normal--24-*-*-*-*-*-iso8859-1,-jis-fixed-medium-r-normal--24-*-*-*-*-*-jisx0208.1983-1");
  */
  font_question = gdk_fontset_load("-adobe-helvetica-medium-r-normal--24-*-*-*-*-*-iso8859-1,-jis-fixed-medium-r-normal--24-*-*-*-*-*-jisx0208.1983-1,*");

}

void draw_question(GdkPixmap *pixmap, GdkGC *gc, type_question* q)
{

  gint x = q->x - 40;
  gint y = q->y - 35;

  if(strcmp(q->phonogram->str,"") == 0)
    return;

  if(q->dflag == DFLAG_NORMAL){
    gdk_gc_set_clip_mask(gc, bit_board[0]);
    gdk_gc_set_clip_origin(gc, x, y);
    gdk_draw_pixmap(pixmap,
		    gc,
		    pix_board[0],
		    0, 0,
		    x, y,
		    im_board[0]->width, im_board[0]->height);
    gdk_gc_set_clip_mask(gc, NULL);

    gdk_draw_string(pixmap, font_question, gc, q->x, q->y, q->ideogram->str);    
    gdk_draw_string(pixmap, font_question, gc, q->x, q->y + 30, q->phonogram->str);    
  }
  else if(q->dflag == DFLAG_MY_SCORE){
    gdk_gc_set_clip_mask(gc, bit_board[1]);
    gdk_gc_set_clip_origin(gc, x, y);
    gdk_draw_pixmap(pixmap,
		    gc,
		    pix_board[1],
		    0, 0,
		    x, y,
		    im_board[1]->width, im_board[1]->height);
    gdk_gc_set_clip_mask(gc, NULL);

    gdk_draw_string(pixmap, font_question, gc, q->x, q->y, q->ideogram->str);    
    gdk_draw_string(pixmap, font_question, gc, q->x, q->y + 30, q->phonogram->str);    
  }
  else{
    gdk_gc_set_clip_mask(gc, bit_board[2]);
    gdk_gc_set_clip_origin(gc, x, y);
    gdk_draw_pixmap(pixmap,
		    gc,
		    pix_board[2],
		    0, 0,
		    x, y,
		    im_board[2]->width, im_board[2]->height);
    gdk_gc_set_clip_mask(gc, NULL);

    gdk_draw_string(pixmap, font_question, gc, q->x, q->y, q->ideogram->str);    
    gdk_draw_string(pixmap, font_question, gc, q->x, q->y + 30, q->phonogram->str);    
  }

}

static void draw_question_list(GdkPixmap *pixmap, GdkGC *gc, type_status *status)
{
  const GSList *list = status->qlist;
  GSList *node = NULL;

  for(node = (GSList *)list; node; node = node->next){
    if(node->data){
      draw_question(pixmap, gc, (type_question *)node->data);
    }
  }

  status->qlist_drawn = TRUE;
}


gint draw_questions(gpointer data)
{

  GtkWidget* area = (GtkWidget *)data;
  gint width = area->allocation.width;
  gint height = area->allocation.height;
  GdkRectangle update_rect;
  GdkGC *gc = area->style->fg_gc[GTK_WIDGET_STATE(area)];
  GdkGC *gc_score = area->style->black_gc;
  GdkGC *gc_string = area->style->black_gc;
  gchar score[64];
  type_status* status = client_status;


  if(anime_list || !status->qlist_drawn){
    /*
      Clear Question Drawing Area 
    */

    /*
      gdk_draw_rectangle(pix_darea,
      area->style->white_gc,
      TRUE,
      0, 0,
      area->allocation.width,
      QAREA_MAX);
    */

    gdk_draw_pixmap(pix_darea,
 		    area->style->white_gc,
		    pix_wall,
		    0, 0,
		    0, 0,
		    width, QAREA_MAX);

    /*
      Draw Questions
    */
    draw_question_list(pix_darea, gc, status);


    /*
      Draw Animations
    */
    anime_list = draw_animation(anime_list, pix_darea, gc, status);


    /*
      Draw Score
    */
    sprintf(score, "SCORE : %10d", 
	    GPOINTER_TO_INT(g_hash_table_lookup(status->score_tbl, 
						status->user_name->str)));
    gdk_draw_string(pix_darea, font_question, gc_score, 400, 25, score);    

  }

  if(status->pressed_keys){
    gdk_draw_pixmap(pix_darea,
		    gc,
		    pix_wall,
		    0, QAREA_MAX,
		    0, QAREA_MAX,
		    width, height - QAREA_MAX);

    /*
      Draw Answer
    */
    if(status->pressed_keys)
      gdk_draw_string(pix_darea, font_question, gc_string, 20, QAREA_MAX + 30, status->pressed_keys->str);
  }

  update_rect.x = 0;
  update_rect.y = 0;
  update_rect.width = width;
  update_rect.height = height;
  
  gtk_widget_draw(area, &update_rect);


  return TRUE;
}


void draw_set_animation(type_status *status, GString *user_name, GString *string)
{ 
  type_question *q = NULL;
  type_actor *actor = NULL;

  // ǳݤ question ǡξõ draw_animation Ǽưõ
  q = type_question_dup(type_status_find_question(status, string));

  if(strcmp(status->user_name->str, user_name->str) == 0){
    //g_print("set carrycat animation\n");
    // ǳݤ acotr ǡξõ draw_animation Ǽưõ
    actor = type_actor_dup(&act_tmp_carrycat);
    actor->x = q->x + 440;
    actor->y = q->y - 70;
    q->dflag = DFLAG_MY_SCORE;
  }else{
    //g_print("set stealcat animation\n");
    // ǳݤ acotr ǡξõ draw_animation Ǽưõ
    actor = type_actor_dup(&act_tmp_stealcat);
    actor->x = q->x - 100;
    actor->y = q->y - 70;
    q->dflag = DFLAG_NOT_MY_SCORE;
  }

  actor->children = g_slist_append(actor->children, q);

  anime_list = append_animate(anime_list, &actor);
}


void
draw_wait_time(GtkWidget *area, GString *time)
{
  gint width = area->allocation.width;
  gint height = area->allocation.height;
  GdkRectangle update_rect;
  GdkGC *gc = area->style->bg_gc[GTK_WIDGET_STATE(area)];
  GdkGC *gc_string = area->style->black_gc;

  gdk_draw_rectangle(area->window,
 		     gc,
		     TRUE, 
		     0, 0,
		     width, height);

  /*
    Draw Time
  */
  gdk_draw_string(area->window, font_question, gc_string,
		  width/2 - 12, 
		  height/2 - 12, 
		  time->str);



  /*
    Draw 
  */
  update_rect.x = 0;
  update_rect.y = QAREA_MAX;
  update_rect.width = width;
  update_rect.height = height - QAREA_MAX;

  gtk_widget_draw(area, &update_rect);
}

