/*
  netype (Sample NTS Client) net.c

  Copyright (C) 2000 Raiko FUNAKAMI

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <sys/types.h>
#include <sys/socket.h>
#include <sys/un.h>
#include <netinet/in.h>


#include <stdio.h>
#include <netdb.h>
#include <unistd.h>
#include <string.h>

#include "gekic.h"
#include "net.h"

extern GtkWidget* win_connection_error_dialog;
extern GtkWidget* win_wait;
extern GtkWidget* label_cerror_dialog;
extern type_status *client_status;


type_net_info *net_init(gchar* hostname, int port)
{
  type_net_info* net_info = NULL;
  struct hostent *hp;
  struct sockaddr_in address;

  net_info = g_malloc(sizeof(type_net_info));
  if(!net_info){
    g_print("Cannot Alloc net_info");
    return NULL;
  }

  net_info->send_msg = g_string_new("");
  net_info->receive_msg = g_string_new("");

  
  if((hp = gethostbyname(hostname)) == NULL){
    g_free(net_info);
    g_print("%s: unknown host.\n", hostname);
    return NULL;
  }
  
  if((net_info->socket = socket(PF_INET, SOCK_STREAM, 0)) < 0){
    g_free(net_info);
    g_print("client: socket error");
    return NULL;
  }
  
  address.sin_family = AF_INET;
  address.sin_port = htons(port);
  
  memcpy(&address.sin_addr, hp->h_addr_list[0], sizeof(address.sin_addr));
  
  if(connect(net_info->socket, &address, sizeof(address)) < 0){
    g_print("client: connection error");
    return NULL;
  }
  
  return net_info;
}


int net_close(type_net_info *net_info)
{
  int answer = FALSE;

  if(net_info && net_info->socket >= 0){
    answer = close(net_info->socket);
  }

  return answer;
}

void type_net_info_free(type_net_info *net_info)
{
  g_string_free(net_info->send_msg, TRUE);
  g_string_free(net_info->receive_msg, TRUE);
  g_free(net_info);
}


void net_write(type_net_info *net_info, char* string)
{
  if(net_info->socket >= 0 && net_info && string){
    g_string_append(net_info->send_msg, string);
  }
}


void net_writeln(type_net_info *net_info, char* string)
{
  int n = 0;
  if(net_info && net_info->socket && net_info->send_msg && string){
    net_write(net_info, string);
    g_string_append(net_info->send_msg, "\n");
    do{
      n += write(net_info->socket, &net_info->send_msg->str[n], net_info->send_msg->len - n);
    }while(n != net_info->send_msg->len);
  }
}

void net_flush(type_net_info *net_info)
{
  int n = 0;
  if(net_info && net_info->socket && net_info->send_msg){
    do{
      n += write(net_info->socket, &net_info->send_msg->str[n], net_info->send_msg->len - n);
    }while(n != net_info->send_msg->len);
  }
}

/**
   sockǡɤ gstr ɲä

   ֤ : ɤ߹ǡ֤
   
*/
int net_read(type_net_info *net_info)
{
  static char buffer[BUFSIZ];
  int n = read(net_info->socket, buffer, BUFSIZ - 2); 
  if(n < 0) {
    g_print("ERROR: net_read, perhaps socket is closed!");
    return -1;
  }
  else if(n){
    buffer[n] = '\0';
    g_string_append(net_info->receive_msg, buffer);
    //fprintf(stderr, "READ MESSAGE:%s\n", buffer);
  }

  return n;
}


/*
  server ³
  user_name : 桼̾
  passwd    : ѥ
  port    : ѥ

  ֤  ³å
*/
  
type_net_info *connect_to_server(gchar* server, gchar* user_name, gchar* passwd, int port)
{
  type_net_info *net_info;
  static GString *buffer = NULL;

  if(!buffer) buffer = g_string_new("CONNECT ");
  else g_string_assign(buffer, "CONNECT ");


  /* ³ޥɺ */
  if(!user_name)
    return NULL;

  g_string_append(buffer, user_name);
  g_string_append(buffer, " ");

  if(passwd)
    g_string_append(buffer, passwd);

  net_info = net_init(server, port);
  if(!net_info){
    fprintf(stderr, "ERROR Cannot connect server!");
    return NULL;
  }

  /* 
     Send Username and Passwd
  */
  net_writeln(net_info, buffer->str);

  return net_info;
}



/*
  source  1ʬΥǡ str ꤹ
  strꤵ줿ʬΥǡ source
  1ʬΥǡ¸ߤʤˤϤʤˤ⤷ʤ

  ֤ : 1ʬΥǡ¸ߤʤˤ FALSE 
*/
static gboolean getline_from_gstring(GString *source, GString *str)
{
  static GString *buffer = NULL;
  char *end = NULL;

  if(!buffer) buffer = g_string_new("");


  if(!source) return FALSE;
  if(!source->str) return FALSE;
  if(!str) return FALSE;
  if(!str->str) return FALSE;



  g_string_assign(buffer, source->str);
  end   = strchr(buffer->str, '\n');

  if(end){
    *end = '\0';
    g_string_append(str, buffer->str);
    g_string_assign(source, end + 1);

    return TRUE;
  }else{

    return FALSE;
  }
}



/*
  data :   
  socket : Ф̿륽å
  condition : 

*/
void read_from_server(gpointer data, int socket, GdkInputCondition condition)
{
  type_status *status = (type_status*) data;
  type_net_info *net_info = NULL;
  static GString *line = NULL;


  if(!status) return;
  if(!status->net_info) return;

  net_info = status->net_info;

  if(!line){
    line = g_string_new("");
  }
  if(socket < 0 || !line)  return;


  if(net_read(net_info) > 0){
    while(getline_from_gstring(net_info->receive_msg, line)){
      chk_command(line, status);
      g_string_assign(line, "");
	
      if(status->end){
	//g_print("END FLAG IS TRUE\n");
	return;
      }
    }
  }
  else{
    if(client_status->accept == TRUE)
      gtk_label_set_text(GTK_LABEL(label_cerror_dialog), "CONNECT CLOSED ERROR!");
    else
      gtk_label_set_text(GTK_LABEL(label_cerror_dialog), "CANNOT USE YOUR USER NAME!");

    gtk_widget_hide(win_wait);
    gtk_widget_show(win_connection_error_dialog);
    
    /*
      Reset Status
    */
    reset_status(client_status);
  }

  g_string_assign(line, "");
}

