/*
  netype (Sample NTS Client) state.c

  Copyright (C) 2000 Raiko FUNAKAMI

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif


#include <stdlib.h>
#include <string.h>
#include "status.h"

typedef struct{
  gint x;
  gint y;
}vertex;


static guint hash_function(gconstpointer key)
{
  const char *s_key;
  guint h_value = 0;
  int index;

  s_key = key;

  if(key == NULL) return 0;
  
  for(index = 0; index <strlen(s_key); index++){
    h_value = (h_value << 4) + (h_value ^ (guint)s_key[index]);
  }

  return h_value;
}

static gint hash_compare(gconstpointer p1, gconstpointer p2)
{
  return (!strcmp((char*)p1, (char*)p2));
}


static void print_question(type_status *status)
{
  /*
  GSList *node;
  g_print("----------Question----------\n");
  for(node = status->qlist; node; node = node->next)
    g_print("%s\n", ((type_question *)node->data)->phonogram->str);
  */
}


type_question* type_question_new(GString* phonogram, GString* ideogram)
{
  type_question *q = (type_question *)g_malloc(sizeof(type_question));

  if(phonogram == NULL)  q->phonogram = g_string_new("");
  else q->phonogram = g_string_new(phonogram->str);

  if(ideogram == NULL) q->ideogram = g_string_new("");
  else q->ideogram = g_string_new(ideogram->str);

  q->x = 0;
  q->y = 0;
  q->dflag = DFLAG_NORMAL;

  return q;
}
type_question* type_question_dup(type_question* old_q)
{
  type_question *q = type_question_new(old_q->phonogram, old_q->ideogram);

  q->x = old_q->x;
  q->y = old_q->y;
  q->dflag = old_q->dflag;

  return q;
}

void type_question_free(type_question* q)
{
  g_string_free(q->phonogram, TRUE);
  g_string_free(q->ideogram, TRUE);
  g_free(q);
}

type_status* type_status_new()
{
  type_status* status = (type_status *)g_malloc(sizeof(type_status));

  status->user_name = g_string_new("");
  status->passwd = g_string_new("");
  status->host_name = g_string_new("");
  status->port = 0;

  status->accept = FALSE;
  status->start = FALSE;
  status->end = FALSE;

  status->pressed_keys = g_string_new("");


  status->qlist = NULL;
  status->qlist_drawn = FALSE;

  status->net_info = NULL;


  status->score_tbl = g_hash_table_new(hash_function, hash_compare);

  return status;
}

void type_status_free(type_status *p)
{
  g_string_free(p->user_name, TRUE);
  g_string_free(p->pressed_keys, TRUE);

  g_hash_table_destroy(p->score_tbl);

  g_free(p);
}




static vertex next_question_vertex(type_status *status)
{
  vertex v;
  static gint i = 0;
  if(i > 5) i = 0;

  v.x = 120;
  v.y = i * 90 + 70;
  i++;

  return v;
}



type_question* type_status_find_question(type_status *status, GString *phonogram)
{
  GSList *node = NULL;
  type_question *q = NULL;

  for(node = status->qlist; node; node = node->next){
    if(strcmp(((type_question *)node->data)->phonogram->str, phonogram->str) == 0)
      q = (type_question *)node->data;
  }

  return q;
}


void type_status_append_question(type_status *status, GString *phonogram, GString *ideogram)
{
  type_question *q = NULL;
  static GString *str_empty = NULL;

  if(!str_empty) str_empty = g_string_new("");
  else g_string_assign(str_empty,"");


  q = type_status_find_question(status, str_empty);
  if(!q){
    /* ˾õ줿̵꤬Ͽΰ */
    vertex v = next_question_vertex(status);
    q = type_question_new(phonogram, ideogram);
    q->x = v.x;
    q->y = v.y;

    /*
      g_print("%d, %d\n", v.x, v.y);
      g_print("append : phonogram = %s, ideogram = %s\n", q->phonogram->str, q->ideogram->str);
    */

    /*
      ɲäΤϿ
     */
    status->qlist = g_slist_append(status->qlist, q);
  }
  else{
    g_string_assign(q->phonogram, phonogram->str);
    g_string_assign(q->ideogram, ideogram->str);
    
    /*
      g_print("assinge : phonogram = %s, ideogram = %s\n", q->phonogram->str, q->ideogram->str);
    */

    /*
      ϿƤΤϿϤʤ
     */
  }



  status->qlist_drawn = FALSE;
}


gboolean type_status_remove_question(type_status *status, GString *phonogram)
{
  type_question *q = NULL;

  q = type_status_find_question(status, phonogram);
  g_print("remove : phonogram = %s\n", q->phonogram->str);
  if(q){
    g_string_assign(q->phonogram, "");
    g_string_assign(q->ideogram, "");
    status->qlist_drawn = FALSE;

    print_question(status);    
    return TRUE;
  }

  return FALSE;
}


void
type_status_remove_all_questions(type_status *status)
{
  GSList *node = NULL;
  type_question *q = NULL;

  for(node = status->qlist; node; node = node->next){
    q = (type_question *)node->data;
    g_string_assign(q->phonogram, "");
    g_string_assign(q->ideogram, "");
  }

  print_question(status);    

  status->qlist_drawn = FALSE;
}


type_rank* type_rank_new(char* name, int score)
{
  type_rank *p;

  p = g_malloc(sizeof(type_rank));
  p->name = g_string_new(name);
  p->score = score;

  return p;
}
void type_rank_free(type_rank* p)
{
  g_string_free(p->name, TRUE);
  g_free(p);
}

